@@ -5,12 +5,12 @@ loss.grpreg <- function(y, yhat, family) {
   } else if (family=="binomial") {
     if (is.matrix(yhat)) {
       val <- matrix(NA, nrow=nrow(yhat), ncol=ncol(yhat))
-      val[y==1,] <- -2*log(yhat[y==1, , drop=FALSE])
-      val[y==0,] <- -2*log(1-yhat[y==0, , drop=FALSE])      
+      if (sum(y==1)) val[y==1,] <- -2*log(yhat[y==1, , drop=FALSE])
+      if (sum(y==0)) val[y==0,] <- -2*log(1-yhat[y==0, , drop=FALSE])      
     } else {
       val <- numeric(length(y))
-      val[y==1] <- -2*log(yhat[y==1])
-      val[y==0] <- -2*log(1-yhat[y==0])
+      if (sum(y==1)) val[y==1] <- -2*log(yhat[y==1])
+      if (sum(y==0)) val[y==0] <- -2*log(1-yhat[y==0])
     }
   } else if (family=="poisson") {
     yly <- y*log(y)
