@@ -1,4 +1,8 @@
-predict.grpreg <- function(object, X, lambda, which=1:length(object$lambda), type=c("link", "response", "class", "coefficients", "vars", "groups", "nvars", "ngroups", "norm"), ...) {
+predict.grpreg <- function(object, X, type=c("link", "response", "class", "coefficients", "vars", "groups", "nvars", "ngroups", "norm"), lambda, which=1:length(object$lambda), ...) {
+  if (!missing(X) && class(X)=="character") {
+    type <- X
+    X <- NULL
+  }
   type <- match.arg(type)
   beta <- coef.grpreg(object, lambda=lambda, which=which, drop=FALSE)
   if (type=="coefficients") return(beta)
@@ -14,7 +18,7 @@ predict.grpreg <- function(object, X, lambda, which=1:length(object$lambda), typ
       return(sapply(g, length))
     }
     if (type=="norm") return(drop(apply(beta[-1, , drop=FALSE], 2, function(x) tapply(x, object$group, function(x){sqrt(sum(x^2))}))))
-    if (missing(X)) stop("Must supply X")
+    if (missing(X) | is.null(X)) stop("Must supply X")
     eta <- sweep(X %*% beta[-1,,drop=FALSE], 2, beta[1,], "+")
     if (object$family=="gaussian" & type=="class") stop("type='class' is not applicable for family='gaussian'")
     if (object$family=="gaussian" | type=="link") return(drop(eta))
